import hashlib

from django.db import models


class UrlManager(models.Manager):
    def get_or_create(self, long_url):
        if self.filter(long_url=long_url).exists():  # If a shortened URL already exists, don't make a duplicate.
            return self.get(long_url=long_url), False

        try:
            hashed = hashlib.sha3_256(long_url.encode('utf-8')).hexdigest()
        except AttributeError:
            hashed = hashlib.sha256(long_url.encode('utf-8')).hexdigest()
        length = 5

        while self.filter(short_id=hashed[:length]).exists():
            length += 1

        url = self.create(short_id=hashed[:length], long_url=long_url)
        url.save()
        return url, True



from django.shortcuts import get_object_or_404, redirect
from django.views.generic.base import View

from shortener.models import Url


class RedirectView(View):
    def get(self, request, short):
        url = get_object_or_404(Url, short_id=short)
        return redirect(url.long_url)



from django.db import models

from shortener.manager import UrlManager


class Url(models.Model):
    long_url = models.URLField()
    short_id = models.SlugField()

    def __str__(self):
        return '%s -- %s' % (self.long_url, self.short_id)

    objects = UrlManager()
